function varargout=cssGUI(profile,varargin)
%%% This GUI implements the comprehensive shear stress method.
%%% If you find this code useful, please cite:
%%% Womack, K.M., Meneveau, C., & Schultz, M.P. (2019). Comprehensive shear stress analysis of turbulent boundary layer profiles. Journal of Fluid Mechanics, 879, 360-389. doi:10.1017/jfm.2019.673
%%% Freely modify this code for your applications. Please maintain the
%%% citation information. Address coorespondence to womack@jhu.edu.

p = inputParser;
% Default initialization
cssRangeInit = [0.15 0.30];
linRangeInit = [0.07 0.15];
utauRangeInit = [0.8 1.2].*profile.utau;
z0RangeInit = [0.5 1.5].*profile.z0;
if profile.d == 0
    dRangeInit = [0 .001];
else
    dRangeInit = [-1 1].*profile.d + profile.d;
end
fullAutoInit = false;
fix_d_init = false;

addRequired(p,'profile',@(x) isa(x,'profileClass'));
addParameter(p,'cssRange',cssRangeInit,@(x)validateattributes(x,{'numeric'},{'numel',2,'increasing','>=',0,'<=',1}));
% addParameter(p,'linRange',linRangeInit,@(x)validateattributes(x,{'numeric'},{'numel',2},{'increasing'},{'>=',0},{'<=',1}));
addParameter(p,'linRange',linRangeInit,@(x)validateattributes(x,{'numeric'},{'numel',2,'increasing','>=',0,'<=',1}));

addParameter(p,'utauRange',utauRangeInit,@(x)validateattributes(x,{'numeric'},{'numel',2,'increasing','>=',0}));
addParameter(p,'z0Range',z0RangeInit,@(x)validateattributes(x,{'numeric'},{'numel',2,'increasing','>=',0}));
addParameter(p,'dRange',dRangeInit,@(x)validateattributes(x,{'numeric'},{'numel',2,'increasing','>=',0}));
addParameter(p,'FullAuto',fullAutoInit,@islogical);
addParameter(p,'FixD',fix_d_init,@islogical);

parse(p,profile,varargin{:});

GUI.linRange = p.Results.linRange;
GUI.cssInd = p.Results.cssRange;
fullAuto = p.Results.FullAuto;
fix_d = p.Results.FixD;

if p.Results.utauRange(1) > profile.utau || profile.utau > p.Results.utauRange(2)
    error('Initial utau is not between utauRange');
end

if p.Results.z0Range(1) > profile.z0 || profile.z0 > p.Results.z0Range(2)
    error('Initial z0 is not between z0Range');
end

if p.Results.dRange(1) > profile.d || profile.d > p.Results.dRange(2)
    error('Initial d is not between dRange');
end

GUI.profile = p.Results.profile;

%{
GUI.profile = varargin{1};
if nargin > 1
    GUI.linRange = varargin{2};
else
    GUI.linRange = [0.07 0.15]; % Sets default linear range
end
if nargin > 2
    GUI.cssInd = varargin{3}; % Sets default CSS range
else
    GUI.cssInd = [0.15 0.30]; % Sets default CSS range
end
if nargin > 3
    fullAuto = varargin{4};
else
    fullAuto = false;
end
if nargin > 4
    fix_d = varargin{5};
else
    fix_d = false;
end
%}

screenSize = get(0,'MonitorPositions');
if size(screenSize,1) == 1
    posGUIfig = [10 40 1366 768];
else
    posGUIfig = [screenSize(1,1)+100 screenSize(1,2)+50 1366 768];
end

GUI.GUIfig = figure('Units','pixels',...
                    'InvertHardcopy','off',...
                    'PaperPositionMode','auto',...
                    'Position',posGUIfig,...
                    'name','CSS GUI - Womack et al. (2019)',...
                    'numbertitle','off',...
                    'MenuBar','figure',...
                    'ToolBar','none',...
                    'resize','off',...
                    'CloseRequestFcn',@GUIfigClosereq);

if any(isnan(GUI.profile.u.mean))
    nanMask = ~isnan(GUI.profile.u.mean);
    nMask = length(nanMask);
    if nMask == length(GUI.profile.z)           GUI.profile.z = GUI.profile.z(nanMask);                     end
    if nMask == length(GUI.profile.u.mean)      GUI.profile.u.mean = GUI.profile.u.mean(nanMask);           end
    if nMask == length(GUI.profile.u.rms)       GUI.profile.u.rms = GUI.profile.u.rms(nanMask);             end
    if nMask == length(GUI.profile.u.skewness)  GUI.profile.u.skewness = GUI.profile.u.skewness(nanMask);   end
    if nMask == length(GUI.profile.u.flatness)  GUI.profile.u.flatness = GUI.profile.u.flatness(nanMask);   end
    if nMask == length(GUI.profile.u.n)         GUI.profile.u.n = GUI.profile.u.n(nanMask);                 end
    if nMask == length(GUI.profile.w.mean)      GUI.profile.w.mean = GUI.profile.w.mean(nanMask);           end
    if nMask == length(GUI.profile.w.rms)       GUI.profile.w.rms = GUI.profile.w.rms(nanMask);             end
    if nMask == length(GUI.profile.w.skewness)  GUI.profile.w.skewness = GUI.profile.w.skewness(nanMask);   end
    if nMask == length(GUI.profile.w.flatness)  GUI.profile.w.flatness = GUI.profile.w.flatness(nanMask);   end
    if nMask == length(GUI.profile.w.n)         GUI.profile.w.n = GUI.profile.w.n(nanMask);                 end
    if nMask == length(GUI.profile.uw)          GUI.profile.uw = GUI.profile.uw(nanMask);                   end
    if nMask == length(GUI.profile.nUW)         GUI.profile.nUW = GUI.profile.nUW(nanMask);                 end
    if nMask == length(GUI.profile.DSS)         GUI.profile.DSS = GUI.profile.DSS(nanMask);                 end
    if nMask == length(GUI.profile.nCoinc)      GUI.profile.nCoinc = GUI.profile.nCoinc(nanMask);           end
end

if ~isempty(GUI.profile.name)
    set(GUI.GUIfig,'name',['CSS GUI - Womack et al. (2019) - ' strrep(strrep(GUI.profile.name,'_',' '),',','.')]);
end
if isempty(GUI.profile.Pi)
    calcPi();
end
if isempty(GUI.profile.Kp)
    GUI.profile.Kp = 0;
end
GUI.profile.theta = calcTheta(GUI.profile,'linear');
GUI.profile.deltaStar = calcDeltaStar(GUI.profile,'linear');
GUI.DSSexist = isprop(GUI.profile,'DSS') && ~isempty(GUI.profile.DSS);

GUI.GUIaxes = axes('Parent',GUI.GUIfig,...
                   'Units','pixels',...
                   'XTick',[0.1 1 10 100 1000],...
                   'Xscale','log','Yscale','lin',...
                   'Box','on',...
                   'NextPlot','replacechildren',...
                   'Position',[74 454 400 300],...
                   'FontSize',12,...
                   'YLimMode','auto');
xlabel(GUI.GUIaxes,'$$(z-d)^+$$','interpreter','latex','FontSize',18);
ylabel(GUI.GUIaxes,'$$u^+$$','interpreter','latex','FontSize',18);
GUI.GUIaxesLegend = legend(GUI.GUIaxes,'Location','northwest');

GUI.GUI_momentumDef = axes('Parent',GUI.GUIfig,...
                           'Units','pixels',...
                           'XTick',[0.1 1 10 100 1000],...
                           'Xscale','log','Yscale','lin',...
                           'Box','on',...
                           'NextPlot','replacechildren',...
                           'Position',[74 71 400 300],...
                           'FontSize',12);
xlabel(GUI.GUI_momentumDef,'$$(z-d)/(\delta-d)$$','interpreter','latex','FontSize',18);
ylabel(GUI.GUI_momentumDef,'$$u^+_\infty-u^+$$','interpreter','latex','FontSize',18);

GUI.GUI_RSS = axes('Parent',GUI.GUIfig,...
                   'Units','pixels',...
                   'Xscale','lin','Yscale','lin',...
                   'Box','on',...
                   'NextPlot','replacechildren',...
                   'Position',[946 454 400 300],...
                   'FontSize',12);
xlabel(GUI.GUI_RSS,'$$(z-d)/(\delta-d)$$','interpreter','latex','FontSize',18);
ylabel(GUI.GUI_RSS,'$$-(u''w'')^+$$','interpreter','latex','FontSize',18);
ylim(GUI.GUI_RSS,[0 1.1]);
GUI.GUI_RSS_Legend = legend(GUI.GUI_RSS,'Location','southwest');

GUI.GUI_linDiagnostic = axes('Parent',GUI.GUIfig,...
                             'Units','pixels',...
                             'Xscale','log','Yscale','lin',...
                             'Box','on',...
                             'NextPlot','replacechildren',...
                             'Position',[528,46,224,168],...
                             'FontSize',8.5);
xlabel(GUI.GUI_linDiagnostic,'$$(z-d)/(\delta-d)$$','interpreter','latex','FontSize',14);
ylabel(GUI.GUI_linDiagnostic,'$$\Phi$$','interpreter','latex','FontSize',14);

GUI.GUI_totalShear = axes('Parent',GUI.GUIfig,...
                          'Units','pixels',...
                          'XTick',[.1 .2 .3 .6 1.0],...
                          'Xscale','log','Yscale','lin',...
                          'Box','on',...
                          'NextPlot','replacechildren',...
                          'Position',[834+344+36,46,132,99],...
                          'FontSize',8.5,...
                          'XLim',[GUI.profile.z(1) GUI.profile.z(length(GUI.profile.z))]./GUI.profile.delta);
xlabel(GUI.GUI_totalShear,'$$(z-d)/(\delta-d)$$','interpreter','latex','FontSize',14);
ylabel(GUI.GUI_totalShear,'$$\tau$$','interpreter','latex','FontSize',14);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Control Group %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
equationOrigin = [498 698];
GUI.equation_text = annotation(GUI.GUIfig,'textbox',...
                                          'Interpreter','latex',...
                                          'Units','pixels',...
                                          'Position',[equationOrigin(1) equationOrigin(2) 370 70],...
                                          'EdgeColor',get(GUI.GUIfig,'Color'),...
                                          'FontSize',14,...
                                          'FontWeight','bold',...
                                          'HorizontalAlignment','center',...
                                          'VerticalAlignment','middle',...
                                          'String','$$\frac{u}{{{u_\tau }}} = \frac{1}{\kappa }\ln \left( {\frac{{z - d}}{{{z_0}}}} \right) + \frac{{2 \Pi }}{\kappa } {W}\left( {\frac{{z - d}}{\delta - d}} \right)$$');

utauGroupOrigin = [equationOrigin(1) equationOrigin(2)-80];
utau_textPos = [utauGroupOrigin(1) utauGroupOrigin(2)+40 80 30];
utau_sliderPos = [utauGroupOrigin(1)+90 utauGroupOrigin(2) 190 30];
utau_minPos = [utauGroupOrigin(1) utauGroupOrigin(2) 80 30];
utau_valPos = [utauGroupOrigin(1)+90 utauGroupOrigin(2)+40 190 30];
utau_maxPos = [utauGroupOrigin(1)+290 utauGroupOrigin(2) 80 30];

GUI.utau_text = annotation(GUI.GUIfig,'textbox',...
                                      'Interpreter','latex',...
                                      'Units','pixels',...
                                      'Position',utau_textPos,...
                                      'EdgeColor',get(GUI.GUIfig,'Color'),...
                                      'FontSize',18,...
                                      'FontWeight','bold',...
                                      'HorizontalAlignment','center',...
                                      'VerticalAlignment','middle',...
                                      'String','$$u_\tau = $$');
GUI.utau_slider = uicontrol('Parent',GUI.GUIfig,...
                            'Style','slide',...
                            'Units','pixels',...
                            'Position',utau_sliderPos,...
                            'Min',p.Results.utauRange(1),...
                            'Max',p.Results.utauRange(2),...
                            'Value',GUI.profile.utau);
GUI.utau_editBox(1) = uicontrol('Parent',GUI.GUIfig,...
                                'Style','edit',...
                                'Units','pixels',...
                                'Position',utau_minPos,...
                                'FontSize',12,...
                                'String',num2str(get(GUI.utau_slider,'Min')));
GUI.utau_editBox(2) = uicontrol('Parent',GUI.GUIfig,...
                                'Style','edit',...
                                'Units','pixels',...
                                'Position',utau_valPos,...
                                'FontSize',16,...
                                'String',num2str(get(GUI.utau_slider,'Value')));
GUI.utau_editBox(3) = uicontrol('Parent',GUI.GUIfig,...
                                'Style','edit',...
                                'Units','pixels',...
                                'Position',utau_maxPos,...
                                'FontSize',12,...
                                'String',num2str(get(GUI.utau_slider,'Max')));

set([GUI.utau_editBox(:);GUI.utau_slider],'Callback',@utau_slider_call);  % Shared Callback.
GUI.optimize_css_button = uicontrol('Parent',GUI.GUIfig,...
                                       'Style','pushbutton',...
                                       'Units','pixels',...
                                       'Position',[utauGroupOrigin(1)+290 utauGroupOrigin(2)+40 80 30],...
                                       'FontSize',10,...
                                       'HorizontalAlignment','center',...
                                       'String','Optimize utau',...
                                       'tooltip','Push to optimize utau only',...
                                       'Callback',@optimize_css);

z0GroupOrigin = [equationOrigin(1) equationOrigin(2)-170];
z0_textPos = [z0GroupOrigin(1) z0GroupOrigin(2)+40 80 30];
z0_sliderPos = [z0GroupOrigin(1)+90 z0GroupOrigin(2) 190 30];
z0_minPos = [z0GroupOrigin(1) z0GroupOrigin(2) 80 30];
z0_valPos = [z0GroupOrigin(1)+90 z0GroupOrigin(2)+40 190 30];
z0_maxPos = [z0GroupOrigin(1)+290 z0GroupOrigin(2) 80 30];

GUI.z0_text = annotation(GUI.GUIfig,'textbox',...
                                    'Interpreter','latex',...
                                    'Units','pixels',...
                                    'Position',z0_textPos,...
                                    'EdgeColor',get(GUI.GUIfig,'Color'),...
                                    'FontSize',18,...
                                    'FontWeight','bold',...
                                    'HorizontalAlignment','center',...
                                    'VerticalAlignment','middle',...
                                    'String','$$z_0 = $$');
GUI.z0_slider = uicontrol('Parent',GUI.GUIfig,...
                          'Style','slide',...
                          'Units','pixels',...
                          'Position',z0_sliderPos,...
                          'Min',p.Results.z0Range(1),...
                          'Max',p.Results.z0Range(2),...
                          'Value',GUI.profile.z0);
GUI.z0_editBox(1) = uicontrol('Parent',GUI.GUIfig,...
                              'Style','edit',...
                              'Units','pixels',...
                              'Position',z0_minPos,...
                              'FontSize',12,...
                              'String',num2str(get(GUI.z0_slider,'Min')));
GUI.z0_editBox(2) = uicontrol('Parent',GUI.GUIfig,...
                              'Style','edit',...
                              'Units','pixels',...
                              'Position',z0_valPos,...
                              'FontSize',16,...
                              'String',num2str(get(GUI.z0_slider,'Value')));
GUI.z0_editBox(3) = uicontrol('Parent',GUI.GUIfig,...
                              'Style','edit',...
                              'Units','pixels',...
                              'Position',z0_maxPos,...
                              'FontSize',12,...
                              'String',num2str(get(GUI.z0_slider,'Max')));

set([GUI.z0_editBox(:);GUI.z0_slider],'Callback',@z0_slider_call);  % Shared Callback.
GUI.optimize_z_button = uicontrol('Parent',GUI.GUIfig,...
                                  'Style','pushbutton',...
                                  'Units','pixels',...
                                  'Position',[z0GroupOrigin(1)+290 z0GroupOrigin(2)+40 80 30],...
                                  'FontSize',10,...
                                  'HorizontalAlignment','center',...
                                  'String','Optimize z0',...
                                  'tooltip','Push to optimize z0 only',...
                                  'Callback',@optimize_z);

dGroupOrigin = [equationOrigin(1) equationOrigin(2)-260];
d_textPos = [dGroupOrigin(1) dGroupOrigin(2)+40 80 30];
d_sliderPos = [dGroupOrigin(1)+90 dGroupOrigin(2) 190 30];
d_minPos = [dGroupOrigin(1) dGroupOrigin(2) 80 30];
d_valPos = [dGroupOrigin(1)+90 dGroupOrigin(2)+40 190 30];
d_maxPos = [dGroupOrigin(1)+290 dGroupOrigin(2) 80 30];

GUI.d_text = annotation(GUI.GUIfig,'textbox',...
                                   'Interpreter','latex',...
                                   'Units','pixels',...
                                   'Position',d_textPos,...
                                   'EdgeColor',get(GUI.GUIfig,'Color'),...
                                   'FontSize',18,...
                                   'FontWeight','bold',...
                                   'HorizontalAlignment','center',...
                                   'VerticalAlignment','middle',...
                                   'String','$$d = $$');
GUI.d_slider = uicontrol('Parent',GUI.GUIfig,...
                         'Style','slide',...
                         'Units','pixels',...
                         'Position',d_sliderPos,...
                         'Min',p.Results.dRange(1),...
                         'Max',p.Results.dRange(2),...
                         'Value',GUI.profile.d);
%{
if GUI.profile.d == 0
    set(GUI.d_slider,'Min',0,'Max',.001);
else
    set(GUI.d_slider,'Min',GUI.profile.d-GUI.profile.d,'Max',GUI.profile.d+GUI.profile.d);
end
                         %}
GUI.d_editBox(1) = uicontrol('Parent',GUI.GUIfig,...
                             'Style','edit',...
                             'Units','pixels',...
                             'Position',d_minPos,...
                             'FontSize',12,...
                             'String',num2str(get(GUI.d_slider,'Min')));
GUI.d_editBox(2) = uicontrol('Parent',GUI.GUIfig,...
                             'Style','edit',...
                             'Units','pixels',...
                             'Position',d_valPos,...
                             'FontSize',16,...
                             'String',num2str(get(GUI.d_slider,'Value')));
GUI.d_editBox(3) = uicontrol('Parent',GUI.GUIfig,...
                             'Style','edit',...
                             'Units','pixels',...
                             'Position',d_maxPos,...
                             'FontSize',12,...
                             'String',num2str(get(GUI.d_slider,'Max')));
set([GUI.d_editBox(:);GUI.d_slider],'Callback',@d_slider_call);  % Shared Callback.
GUI.optimize_d_button = uicontrol('Parent',GUI.GUIfig,...
                                  'Style','pushbutton',...
                                  'Units','pixels',...
                                  'Position',[dGroupOrigin(1)+290 dGroupOrigin(2)+40 80 30],...
                                  'FontSize',10,...
                                  'HorizontalAlignment','center',...
                                  'String','Optimize d',...
                                  'tooltip','Push to optimize d only',...
                                  'Callback',@optimize_d);

PiGroupOrigin = [equationOrigin(1) equationOrigin(2)-350];
Pi_textPos = [PiGroupOrigin(1) PiGroupOrigin(2)+40 80 30];
Pi_sliderPos = [PiGroupOrigin(1)+90 PiGroupOrigin(2) 190 30];
Pi_minPos = [PiGroupOrigin(1) PiGroupOrigin(2) 80 30];
Pi_valPos = [PiGroupOrigin(1)+90 PiGroupOrigin(2)+40 190 30];
Pi_maxPos = [PiGroupOrigin(1)+290 PiGroupOrigin(2) 80 30];
Pi_pushbuttonPos = [PiGroupOrigin(1)+290 PiGroupOrigin(2)+40 80 30];
Pi_radioTextPos = [PiGroupOrigin(1) PiGroupOrigin(2)-45 100 30];
Pi_radio1Pos = [PiGroupOrigin(1)+100 PiGroupOrigin(2)-40 90 30];
Pi_radio2Pos = [PiGroupOrigin(1)+190 PiGroupOrigin(2)-40 90 30];
Pi_radio3Pos = [PiGroupOrigin(1)+280 PiGroupOrigin(2)-40 90 30];

GUI.Pi_text = annotation(GUI.GUIfig,'textbox',...
                                    'Interpreter','latex',...
                                    'Units','pixels',...
                                    'Position',Pi_textPos,...
                                    'EdgeColor',get(GUI.GUIfig,'Color'),...
                                    'FontSize',18,...
                                    'FontWeight','bold',...
                                    'HorizontalAlignment','center',...
                                    'VerticalAlignment','middle',...
                                    'String','$$\Pi = $$');
GUI.Pi_slider = uicontrol('Parent',GUI.GUIfig,...
                          'Style','slide',...
                          'Enable','off',...
                          'Visible','off',...
                          'Units','pixels',...
                          'Position',Pi_sliderPos,...
                          'Min',GUI.profile.Pi*.5,...
                          'Max',GUI.profile.Pi*1.5,...
                          'Value',GUI.profile.Pi);
GUI.Pi_editBox(1) = uicontrol('Parent',GUI.GUIfig,...
                              'Style','edit',...
                              'Enable','off',...
                              'Visible','off',...
                              'Units','pixels',...
                              'Position',Pi_minPos,...
                              'FontSize',12,...
                              'String',num2str(get(GUI.Pi_slider,'Min')));
GUI.Pi_editBox(2) = uicontrol('Parent',GUI.GUIfig,...
                              'Style','edit',...
                              'Enable','off',...
                              'Units','pixels',...
                              'Position',Pi_valPos,...
                              'FontSize',16,...
                              'String',num2str(get(GUI.Pi_slider,'Value')));
GUI.Pi_editBox(3) = uicontrol('Parent',GUI.GUIfig,...
                              'Style','edit',...
                              'Enable','off',...
                              'Visible','off',...
                              'Units','pixels',...
                              'Position',Pi_maxPos,...
                              'FontSize',12,...
                              'String',num2str(get(GUI.Pi_slider,'Max')));
GUI.Pi_pushbutton = uicontrol('Parent',GUI.GUIfig,...
                              'Style','pushbutton',...
                              'Enable','off',...
                              'Visible','off',...
                              'Units','pixels',...
                              'Position',Pi_pushbuttonPos,...
                              'FontSize',10,...
                              'FontWeight','normal',...
                              'string','Calculate',...
                              'tooltip','Push to calculate \Pi from current values');
GUI.Pi_radioText = annotation(GUI.GUIfig,'textbox',...
                                         'Interpreter','latex',...
                                         'Units','pixels',...
                                         'Position',Pi_radioTextPos,...
                                         'EdgeColor',get(GUI.GUIfig,'Color'),...
                                         'FontSize',14,...
                                         'FontWeight','bold',...
                                         'HorizontalAlignment','left',...
                                         'VerticalAlignment','bottom',...
                                         'String','$$\Pi$$ options');
GUI.Pi_radio(1) = uicontrol('Parent',GUI.GUIfig,...
                            'Style','radio',...
                            'Units','pixels',...
                            'Position',Pi_radio1Pos,...
                            'FontSize',12,...
                            'FontWeight','normal',...
                            'HorizontalAlignment','left',...
                            'String','Auto',...
                            'Value',0);
GUI.Pi_radio(2) = uicontrol('Parent',GUI.GUIfig,...
                            'Style','radio',...
                            'Units','pixels',...
                            'Position',Pi_radio2Pos,...
                            'FontSize',12,...
                            'FontWeight','normal',...
                            'HorizontalAlignment','left',...
                            'String','Manual',...
                            'Value',0);
GUI.Pi_radio(3) = uicontrol('Parent',GUI.GUIfig,...
                            'Style','radio',...
                            'Units','pixels',...
                            'Position',Pi_radio3Pos,...
                            'FontSize',12,...
                            'FontWeight','normal',...
                            'HorizontalAlignment','left',...
                            'String','Off',...
                            'Value',1);
set([GUI.Pi_editBox(:);GUI.Pi_slider;GUI.Pi_pushbutton;GUI.Pi_radio(:)],'Callback',@Pi_slider_call);  % Shared Callback.

cssInd_textPos = [498 273 120 30];
cssInd_minPos = [cssInd_textPos(1)+130 cssInd_textPos(2) 35 30];
cssInd_maxPos = [cssInd_textPos(1)+185 cssInd_textPos(2) 35 30];
cssInd_colon = [cssInd_textPos(1)+165 cssInd_textPos(2) 20 30];
GUI.cssInd_text = annotation(GUI.GUIfig,'textbox',...
                                           'Interpreter','tex',...
                                           'Units','pixels',...
                                           'Position',cssInd_textPos,...
                                           'EdgeColor',get(GUI.GUIfig,'Color'),...
                                           'FontSize',12,...
                                           'HorizontalAlignment','left',...
                                           'VerticalAlignment','middle',...
                                           'String','CSS range');
GUI.cssInd_editBox(1) = uicontrol('Parent',GUI.GUIfig,...
                                     'Style','edit',...
                                     'Units','pixels',...
                                     'Position',cssInd_minPos,...
                                     'FontSize',10,...
                                     'String',num2str(GUI.cssInd(1)));
GUI.cssInd_colon = annotation(GUI.GUIfig,'textbox',...
                                            'Interpreter','tex',...
                                            'Units','pixels',...
                                            'Position',cssInd_colon,...
                                            'EdgeColor',get(GUI.GUIfig,'Color'),...
                                            'FontSize',14,...
                                            'FontWeight','bold',...
                                            'HorizontalAlignment','center',...
                                            'VerticalAlignment','middle',...
                                            'String','-');
GUI.cssInd_editBox(2) = uicontrol('Parent',GUI.GUIfig,...
                                     'Style','edit',...
                                     'Units','pixels',...
                                     'Position',cssInd_maxPos,...
                                     'FontSize',10,...
                                     'String',num2str(GUI.cssInd(2)));
set(GUI.cssInd_editBox(:),'Callback',@cssInd_call);  % Shared Callback.

linRange_textPos = [498 238 120 30];
linRange_minPos = [linRange_textPos(1)+130 linRange_textPos(2) 35 30];
linRange_maxPos = [linRange_textPos(1)+185 linRange_textPos(2) 35 30];
linRange_colon = [linRange_textPos(1)+165 linRange_textPos(2) 20 30];

GUI.linRange_text = annotation(GUI.GUIfig,'textbox',...
                                          'Interpreter','tex',...
                                          'Units','pixels',...
                                          'Position',linRange_textPos,...
                                          'EdgeColor',get(GUI.GUIfig,'Color'),...
                                          'FontSize',12,...
                                          'HorizontalAlignment','left',...
                                          'VerticalAlignment','middle',...
                                          'String','Linear range');
GUI.linRange_editBox(1) = uicontrol('Parent',GUI.GUIfig,...
                                    'Style','edit',...
                                    'Units','pixels',...
                                    'Position',linRange_minPos,...
                                    'FontSize',10,...
                                    'String',num2str(GUI.linRange(1)));
GUI.linRange_colon = annotation(GUI.GUIfig,'textbox',...
                                           'Interpreter','tex',...
                                           'Units','pixels',...
                                           'Position',linRange_colon,...
                                           'EdgeColor',get(GUI.GUIfig,'Color'),...
                                           'FontSize',14,...
                                           'FontWeight','bold',...
                                           'HorizontalAlignment','center',...
                                           'VerticalAlignment','middle',...
                                           'String','-');
GUI.linRange_editBox(2) = uicontrol('Parent',GUI.GUIfig,...
                                    'Style','edit',...
                                    'Units','pixels',...
                                    'Position',linRange_maxPos,...
                                    'FontSize',10,...
                                    'String',num2str(GUI.linRange(2)));
set(GUI.linRange_editBox(:),'Callback',@linRange_call);  % Shared Callback.

GUI.optimize_dz_button = uicontrol('Parent',GUI.GUIfig,...
                                   'Style','pushbutton',...
                                   'Units','pixels',...
                                   'Position',[linRange_textPos(1)+240 linRange_textPos(2) 120 30],...
                                   'FontSize',10,...
                                   'HorizontalAlignment','center',...
                                   'String','Optimize d and z0',...
                                   'tooltip','Push to optimize d and z0 with a linear regression',...
                                   'Callback',@optimize_dz);
                               
GUI.optimize_auto_button = uicontrol('Parent',GUI.GUIfig,...
                                     'Style','pushbutton',...
                                     'Units','pixels',...
                                     'Position',[linRange_textPos(1)+365 linRange_textPos(2) 80 30],...
                                     'FontSize',10,...
                                     'HorizontalAlignment','center',...
                                     'String','Auto',...
                                     'tooltip','Push to perform automatic iterative optimization to convergence or 10 iterations',...
                                     'Callback',@optimize_auto);

GUI.fix_d_toggle = uicontrol('Parent',GUI.GUIfig,...
                             'Style','togglebutton',...
                             'Units','pixels',...
                             'Position',[linRange_textPos(1)+365 cssInd_textPos(2) 80 30],...
                             'FontSize',10,...
                             'HorizontalAlignment','center',...
                             'String','Fix d',...
                             'tooltip','Toggles automatic iteration to assume current d. Looping iteration optimizes utau then optimizes z0 only when selected.',...
                             'Min',0,...
                             'Max',1,...
                             'Value',fix_d);

GUI.smoothwallAuto_toggle = uicontrol('Parent',GUI.GUIfig,...
                                      'Style','togglebutton',...
                                      'Units','pixels',...
                                      'Position',[linRange_textPos(1)+365 PiGroupOrigin(2)-40 80 30],...
                                      'FontSize',10,...
                                      'HorizontalAlignment','center',...
                                      'String','Smooth auto',...
                                      'tooltip','Toggles automatic iteration to smooth mode assuming d=0. Looping iteration optimizes utau then calculates z0=nu/utau*exp(-A*kvk) only when selected.',...
                                      'Min',0,...
                                      'Max',1,...
                                      'Value',0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Optimization Group %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
optimizeGroupOrigin = [834 46];
GUI.GUI_ls_uz = axes('Parent',GUI.GUIfig,...
                     'Units','pixels',...
                     'Box','on',...
                     'Position',[optimizeGroupOrigin(1)+172,optimizeGroupOrigin(2)+172,168,168],...
                     'FontSize',8.5,...
                     'XTickLabel',[]);
% xlabel(GUI.GUI_ls_uz,'$$z_0$$','interpreter','latex','FontSize',14);
ylabel(GUI.GUI_ls_uz,'$$u_{\tau}$$','interpreter','latex','FontSize',14);

GUI.GUI_ls_du = axes('Parent',GUI.GUIfig,...
                     'Units','pixels',...
                     'Box','on',...
                     'Position',[optimizeGroupOrigin(1),optimizeGroupOrigin(2),168,168],...
                     'FontSize',8.5);
xlabel(GUI.GUI_ls_du,'$$u_{\tau}$$','interpreter','latex','FontSize',14);
ylabel(GUI.GUI_ls_du,'$$d$$','interpreter','latex','FontSize',14);

GUI.GUI_ls_dz = axes('Parent',GUI.GUIfig,...
                     'Units','pixels',...
                     'Box','on',...
                     'Position',[optimizeGroupOrigin(1)+172,optimizeGroupOrigin(2),168,168],...
                     'FontSize',8.5,...
                     'YTickLabel',[]);
xlabel(GUI.GUI_ls_dz,'$$z_0$$','interpreter','latex','FontSize',14);
% ylabel(GUI.GUI_ls_dz,'$$d$$','interpreter','latex','FontSize',14);
% GUI.GUI_ls_dz_colorbar = colorbar(GUI.GUI_ls_dz,'Units','pixels');
                               
GUI.GUI_ls_css = axes('Parent',GUI.GUIfig,...
                         'Units','pixels',...
                         'Box','on',...
                         'Position',[optimizeGroupOrigin(1)+344,optimizeGroupOrigin(2)+172,168,168],...
                         'FontSize',8.5,...
                         'YTickLabel',[]);
xlabel(GUI.GUI_ls_css,'$$z_0$$','interpreter','latex','FontSize',14);
% ylabel(GUI.GUI_ls_css,'$$u_\tau$$','interpreter','latex','FontSize',14);

GUI.LSres = 42; % Changes LS resolution if needed for expense

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Load DNS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% DNS profile data are from J. A. Sillero, J. Jimnez, and R. D. Moser,
% One-point statistics for turbulent wall-bounded flows at Reynolds
% numbers up to ?+ ? 2000, Phys. Fluids 25(10), 105102 (2013). 
% The included data was downloaded on 12 August 2019 from
% https://torroja.dmt.upm.es/turbdata/blayers/high_re/profiles/
% Other profile statistics are available on the website.

DNS = load('DNSprofiles.mat');
for m = 1:6
    GUI.DNS(m).Retau = DNS.val(m).Retau;
    GUI.DNS(m).momentumDef = DNS.val(m).ua(length(DNS.val(m).ua))-DNS.val(m).ua;
    GUI.DNS(m).yd99 = DNS.val(m).yd99;
end
clear DNS;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Execute GUI %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

update_linInd();
plotShear();
calcRMS_dz();
calcRMS_du();
calcRMS_uz();
plotFit();
if fullAuto
    optimize_auto();
    varargout{1} = GUI.profile;
    GUIfigClosereq([],[]);
else
    uiwait(GUI.GUIfig);
    varargout{1} = GUI.profile;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Callback control functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function [] = utau_slider_call(varargin)
        % Callback for the edit box and slider.
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        h = varargin{1};  % Get calling handle and structure.
        SL = get(GUI.utau_slider,{'Min','Value','Max'});  % Get the slider's info.
        E = str2double(get(h,'String'));  % Numerical edit string.
        
        switch h  % Who called?
            case GUI.utau_editBox(1)
                if E <= SL{2}
                    set(GUI.utau_slider,'Min',E)  % E is less than current value.
                elseif E < SL{3}
                    set(GUI.utau_slider,'Value',E,'Min',E) % E is less than max value.
                    set(GUI.utau_editBox(2),'String',E) % Set the current display.
                    GUI.profile.utau = E;
                else
                    set(h,'String',SL{1}) % Reset the value.
                end
                calcRMS_du();
                calcRMS_uz();
            case GUI.utau_editBox(2)
                if E >= SL{1} && E <= SL{3}
                    set(GUI.utau_slider,'Value',E)  % E falls within range of slider.
                    GUI.profile.utau = E;
                else
                    set(h,'String',SL{2}) % User tried to set slider out of range.
                end
                calcRMS_dz();
            case GUI.utau_editBox(3)
                if E >= SL{2}
                    set(GUI.utau_slider,'Max',E)  % E is less than current value.
                elseif E > SL{1}
                    set(GUI.utau_slider,'Value',E,'Max',E) % E is less than max value.
                    set(GUI.utau_editBox(2),'String',E) % Set the current display.
                    GUI.profile.utau = E;
                else
                    set(h,'String',SL{3}) % Reset the value.
                end
                calcRMS_du();
                calcRMS_uz();
            case GUI.utau_slider
                set(GUI.utau_editBox(2),'String',SL{2}) % Set edit to current slider.
                GUI.profile.utau = SL{2};
                calcRMS_dz();
            otherwise
                % Do nothing
        end
        plotShear();
        plotFit();
    end

    function [] = z0_slider_call(varargin)
        % Callback for the edit box and slider.
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        h = varargin{1};  % Get calling handle and structure.
        SL = get(GUI.z0_slider,{'Min','Value','Max'});  % Get the slider's info.
        E = str2double(get(h,'String'));  % Numerical edit string.
        
        switch h  % Who called?
            case GUI.z0_editBox(1)
                if E <= SL{2}
                    set(GUI.z0_slider,'Min',E)  % E is less than current value.
                elseif E < SL{3}
                    set(GUI.z0_slider,'Value',E,'Min',E) % E is less than max value.
                    GUI.profile.z0 = E;
                    set(GUI.z0_editBox(2),'String',E) % Set the current display.
                else
                    set(h,'String',SL{1}) % Reset the value.
                end
                calcRMS_dz();
                calcRMS_uz();
            case GUI.z0_editBox(2)
                if E >= SL{1} && E <= SL{3}
                    set(GUI.z0_slider,'Value',E)  % E falls within range of slider.
                    GUI.profile.z0 = E;
                else
                    set(h,'String',SL{2}) % User tried to set slider out of range.
                end
                calcRMS_du();
            case GUI.z0_editBox(3)
                if E >= SL{2}
                    set(GUI.z0_slider,'Max',E)  % E is less than current value.
                elseif E > SL{1}
                    set(GUI.z0_slider,'Value',E,'Max',E) % E is less than max value.
                    GUI.profile.z0 = E;
                    set(GUI.z0_editBox(2),'String',E) % Set the current display.
                else
                    set(h,'String',SL{3}) % Reset the value.
                end
                calcRMS_dz();
                calcRMS_uz();
            case GUI.z0_slider
                set(GUI.z0_editBox(2),'String',SL{2}) % Set edit to current slider.
                GUI.profile.z0 = SL{2};
                calcRMS_du();
            otherwise
                % Do nothing
        end
        plotFit();
    end

    function [] = d_slider_call(varargin)
        % Callback for the edit box and slider.
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        h = varargin{1};  % Get calling handle and structure.
        SL = get(GUI.d_slider,{'Min','Value','Max'});  % Get the slider's info.
        E = str2double(get(h,'String'));  % Numerical edit string.
        
        switch h  % Who called?
            case GUI.d_editBox(1)
                if E <= SL{2}
                    set(GUI.d_slider,'Min',E)  % E is less than current value.
                elseif E < SL{3}
                    set(GUI.d_slider,'Value',E,'Min',E) % E is less than max value.
                    set(GUI.d_editBox(2),'String',E) % Set the current display.
                    GUI.profile.d = E;
                else
                    set(h,'String',SL{1}) % Reset the value.
                end
                calcRMS_dz();
                calcRMS_du();
            case GUI.d_editBox(2)
                if E >= SL{1} && E <= SL{3}
                    set(GUI.d_slider,'Value',E)  % E falls within range of slider.
                    GUI.profile.d = E;
                else
                    set(h,'String',SL{2}) % User tried to set slider out of range.
                end
                calcRMS_uz();
            case GUI.d_editBox(3)
                if E >= SL{2}
                    set(GUI.d_slider,'Max',E)  % E is less than current value.
                elseif E > SL{1}
                    set(GUI.d_slider,'Value',E,'Max',E) % E is less than max value.
                    set(GUI.d_editBox(2),'String',E) % Set the current display.
                    GUI.profile.d = E;
                else
                    set(h,'String',SL{3}) % Reset the value.
                end
                calcRMS_dz();
                calcRMS_du();
            case GUI.d_slider
                set(GUI.d_editBox(2),'String',SL{2}) % Set edit to current slider.
                GUI.profile.d = SL{2};
                calcRMS_uz();
            otherwise
                % Do nothing
        end
        update_linInd();
        plotShear();
        plotFit();
    end

    function [] = Pi_slider_call(varargin)
        % Callback for the edit box and slider.
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        h = varargin{1};  % Get calling handle and structure.
        SL = get(GUI.Pi_slider,{'Min','Value','Max'});  % Get the slider's info.
        E = str2double(get(h,'String'));  % Numerical edit string.
        
        switch h  % Who called?
            case GUI.Pi_editBox(1)
                if E <= SL{2}
                    set(GUI.Pi_slider,'Min',E)  % E is less than current value.
                elseif E < SL{3}
                    set(GUI.Pi_slider,'Value',E,'Min',E) % E is less than max value.
                    set(GUI.Pi_editBox(2),'String',E) % Set the current display.
                else
                    set(h,'String',SL{1}) % Reset the value.
                end
            case GUI.Pi_editBox(2)
                if E >= SL{1} && E <= SL{3}
                    set(GUI.Pi_slider,'Value',E)  % E falls within range of slider.
                else
                    set(h,'String',SL{2}) % User tried to set slider out of range.
                end
            case GUI.Pi_editBox(3)
                if E >= SL{2}
                    set(GUI.Pi_slider,'Max',E)  % E is less than current value.
                elseif E > SL{1}
                    set(GUI.Pi_slider,'Value',E,'Max',E) % E is less than max value.
                    set(GUI.Pi_editBox(2),'String',E) % Set the current display.
                else
                    set(h,'String',SL{3}) % Reset the value.
                end
            case GUI.Pi_slider
                set(GUI.Pi_editBox(2),'String',SL{2}) % Set edit to current slider.
            case GUI.Pi_radio(1)
                set(GUI.Pi_radio([2,3]),'Value',0)
                set(GUI.Pi_pushbutton,'Enable','off','Visible','off');
                set([GUI.Pi_editBox([1,3]) GUI.Pi_slider],'Enable','off','Visible','off');
                set(GUI.Pi_editBox(2),'Enable','off');
                calcPi();
            case GUI.Pi_radio(2)
                set(GUI.Pi_radio([1,3]),'Value',0)
                set(GUI.Pi_pushbutton,'Enable','on','Visible','on');
                if GUI.profile.Pi < SL{1}
                    GUI.profile.Pi = SL{1};
                    set(GUI.Pi_slider,'Value',SL{1});
                    set(GUI.Pi_editBox(2),'String',num2str(SL{1}));
                elseif GUI.profile.Pi > SL{3}
                    GUI.profile.Pi = SL{3};
                    set(GUI.Pi_slider,'Value',SL{3});
                    set(GUI.Pi_editBox(2),'String',num2str(SL{3}));
                else
                    set(GUI.Pi_slider,'Value',GUI.profile.Pi);
                    set(GUI.Pi_editBox(2),'String',num2str(GUI.profile.Pi));
                end
                set([GUI.Pi_editBox(:);GUI.Pi_slider],'Enable','on','Visible','on');
            case GUI.Pi_radio(3)
                set(GUI.Pi_radio([1,2]),'Value',0)
                set(GUI.Pi_pushbutton,'Enable','off','Visible','off');
                set([GUI.Pi_editBox([1,3]) GUI.Pi_slider],'Enable','off','Visible','off');
                set(GUI.Pi_editBox(2),'Enable','off');
                calcPi();
            case GUI.Pi_pushbutton
                calcPi();
            otherwise
                % Do nothing
        end
        plotFit();
    end

    function [] = linRange_call(varargin)
        % Callback for the edit box and slider.
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        h = varargin{1};  % Get calling handle and structure.
        E = str2double(get(h,'String'));  % Numerical edit string.
        
        switch h  % Who called?
            case GUI.linRange_editBox(1)
                GUI.linRange(1) = E;
                set(h,'String',num2str(E));
            case GUI.linRange_editBox(2)
                GUI.linRange(2) = E;
                set(h,'String',num2str(E));
            otherwise
                % Do nothing
        end
        update_linInd()
        calcRMS_dz();
        calcRMS_du();
        calcRMS_uz();
        plotFit();
    end

    function [] = cssInd_call(varargin)
        % Callback for the edit box and slider.
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        h = varargin{1};  % Get calling handle and structure.
        E = str2double(get(h,'String'));  % Numerical edit string.
        
        switch h  % Who called?
            case GUI.cssInd_editBox(1)
                GUI.cssInd(1) = E;
                set(h,'String',num2str(E));
            case GUI.cssInd_editBox(2)
                GUI.cssInd(2) = E;
                set(h,'String',num2str(E));
            otherwise
                % Do nothing
        end
        plotFit();
    end

    function plotFit()
        % Calculate non-dim profile
        delta_nu = GUI.profile.nu/GUI.profile.utau;
        z_plus = (GUI.profile.z-GUI.profile.d)./delta_nu;
        u_plus = GUI.profile.u.mean./GUI.profile.utau;
        u_plus = u_plus';
        
        % Calculate fit profile
        z_plus_fit = logspace(log10(min(z_plus)),log10(max(z_plus)),100);
        u_plus_fit = 1/GUI.profile.kvk*log(z_plus_fit.*delta_nu./GUI.profile.z0);
        if GUI.Pi_radio(1).Value
            calcPi();
            u_plus_fit = u_plus_fit+2.*GUI.profile.Pi./GUI.profile.kvk.*sin(pi/2*z_plus_fit.*(delta_nu./(GUI.profile.delta-GUI.profile.d))).^2;
            [u_plus_fit_max, I] = max(u_plus_fit);
            u_plus_fit(I:length(u_plus_fit)) = u_plus_fit_max;
        elseif GUI.Pi_radio(2).Value
            u_plus_fit = u_plus_fit+2.*GUI.profile.Pi./GUI.profile.kvk.*sin(pi/2*z_plus_fit.*(delta_nu./(GUI.profile.delta-GUI.profile.d))).^2;
            [u_plus_fit_max, I] = max(u_plus_fit);
            u_plus_fit(I:length(u_plus_fit)) = u_plus_fit_max;
        else
            calcPi();
            % Off radio selected. Do nothing.
        end
        
        % Graph fit profile and data
        plot(GUI.GUIaxes,z_plus,u_plus,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','b','MarkerFaceColor','b');
        set(GUI.GUIaxes,'NextPlot','add');
        plot(GUI.GUIaxes,z_plus(GUI.profile.linInd),u_plus(GUI.profile.linInd),'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','r','MarkerFaceColor','r');
        plot(GUI.GUIaxes,z_plus_fit,u_plus_fit,'k','LineStyle','-','LineWidth',1.2,'Marker','none','MarkerSize',16);
        set(GUI.GUIaxes,'NextPlot','replacechildren');
        xlim(GUI.GUIaxes,[min(z_plus_fit) max(z_plus_fit)]);
        
        % Graph momentum deficit
        Retau = GUI.profile.utau.*(GUI.profile.delta-GUI.profile.d)/GUI.profile.nu;
        ReInd = interp1([GUI.DNS(1:6).Retau],[1:6],Retau,'nearest','extrap');
        
        plot(GUI.GUI_momentumDef,GUI.DNS(ReInd).yd99,GUI.DNS(ReInd).momentumDef,'g','LineWidth',2);
        set(GUI.GUI_momentumDef,'NextPlot','add');
        plot(GUI.GUI_momentumDef,(GUI.profile.z-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d),(GUI.profile.Ue-GUI.profile.u.mean)/GUI.profile.utau,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','b','MarkerFaceColor','b');
        plot(GUI.GUI_momentumDef,(GUI.profile.z(GUI.profile.linInd)-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d),(GUI.profile.Ue-GUI.profile.u.mean(GUI.profile.linInd))/GUI.profile.utau,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','r','MarkerFaceColor','r');
        set(GUI.GUI_momentumDef,'NextPlot','replacechildren');
        xlim(GUI.GUI_momentumDef,[min((GUI.profile.z-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d)) max((GUI.profile.z-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d))]);
        
        % Graph Linear Diagnostic
        plotLinDiagnostic();
        
        % Graph CSS
        plotRSS();
        
        % Load LS data
        utauRange = linspace(GUI.utau_slider.Min,GUI.utau_slider.Max,GUI.LSres);
        z0Range = linspace(GUI.z0_slider.Min,GUI.z0_slider.Max,GUI.LSres);
        dRange = linspace(GUI.d_slider.Min,GUI.d_slider.Max,GUI.LSres);
        minRMS = min(vertcat(reshape(GUI.RMS_dz,[],1),reshape(GUI.RMS_du,[],1),reshape(GUI.RMS_uz,[],1)));
        
        % Plot d vs. z_0 LS data
        caxis(GUI.GUI_ls_dz,[minRMS 10*minRMS]);
        contourf(GUI.GUI_ls_dz,z0Range,dRange,GUI.RMS_dz,linspace(minRMS,10*minRMS,40),'LineStyle','none');
        xlabel(GUI.GUI_ls_dz,'$$z_0$$','interpreter','latex','FontSize',14);
        % ylabel(GUI.GUI_ls_dz,'$$d$$','interpreter','latex','FontSize',14);
        set(GUI.GUI_ls_dz,'NextPlot','add');
        plot(GUI.GUI_ls_dz,[GUI.profile.z0 GUI.profile.z0],[dRange(1) dRange(GUI.LSres)],'r-',[z0Range(1) z0Range(GUI.LSres)],[GUI.profile.d GUI.profile.d],'r-');
        set(GUI.GUI_ls_dz,'NextPlot','replacechildren','YTickLabel',[]);
        
        % Plot d vs. utau LS data
        caxis(GUI.GUI_ls_du,[minRMS 10*minRMS]);
        contourf(GUI.GUI_ls_du,utauRange,dRange,GUI.RMS_du,linspace(minRMS,10*minRMS,40),'LineStyle','none');
        xlabel(GUI.GUI_ls_du,'$$u_{\tau}$$','interpreter','latex','FontSize',14);
        ylabel(GUI.GUI_ls_du,'$$d$$','interpreter','latex','FontSize',14);
        set(GUI.GUI_ls_du,'NextPlot','add');
        plot(GUI.GUI_ls_du,[GUI.profile.utau GUI.profile.utau],[dRange(1) dRange(GUI.LSres)],'r-',[utauRange(1) utauRange(GUI.LSres)],[GUI.profile.d GUI.profile.d],'r-');
        set(GUI.GUI_ls_du,'NextPlot','replacechildren');
        
        % Plot utau vs. z LS data
        caxis(GUI.GUI_ls_uz,[minRMS 10*minRMS]);
        contourf(GUI.GUI_ls_uz,z0Range,utauRange,GUI.RMS_uz,linspace(minRMS,10*minRMS,40),'LineStyle','none');
        % xlabel(GUI.GUI_ls_uz,'$$z_0$$','interpreter','latex','FontSize',14);
        ylabel(GUI.GUI_ls_uz,'$$u_{\tau}$$','interpreter','latex','FontSize',14);
        set(GUI.GUI_ls_uz,'NextPlot','add');
        plot(GUI.GUI_ls_uz,[GUI.profile.z0 GUI.profile.z0],[utauRange(1) utauRange(GUI.LSres)],'r-',[z0Range(1) z0Range(GUI.LSres)],[GUI.profile.utau GUI.profile.utau],'r-');
        set(GUI.GUI_ls_uz,'NextPlot','replacechildren','XTickLabel',[]);
        
        % Plot CSS LS data
        RMScssVec = reshape(GUI.RMS_css,[],1);
        minRMScss = min(RMScssVec);
        maxRMScss = max(RMScssVec);
        cRMScss = [minRMScss 0.1*(maxRMScss-minRMScss)+minRMScss];
        caxis(GUI.GUI_ls_css,cRMScss);
        contourf(GUI.GUI_ls_css,z0Range,utauRange,GUI.RMS_css,linspace(cRMScss(1),cRMScss(2),128),'LineStyle','none');
        xlabel(GUI.GUI_ls_css,'$$z_0$$','interpreter','latex','FontSize',14);
        % ylabel(GUI.GUI_ls_css,'$$u_{\tau}$$','interpreter','latex','FontSize',14);
        set(GUI.GUI_ls_css,'NextPlot','add');
        plot(GUI.GUI_ls_css,[GUI.profile.z0 GUI.profile.z0],[utauRange(1) utauRange(GUI.LSres)],'r-',[z0Range(1) z0Range(GUI.LSres)],[GUI.profile.utau GUI.profile.utau],'r-');
        set(GUI.GUI_ls_css,'NextPlot','replacechildren','YTickLabel',[]);
        
        GUI.GUIaxesLegend.String = {'Profile','Linear range','Fit'}; % Some legends must be reset due to error
        legend(GUI.GUI_momentumDef,['DNS Re_{\tau}=' num2str(round(GUI.DNS(ReInd).Retau))],['Profile Re_{\tau}=' num2str(round(GUI.profile.utau*((GUI.profile.delta-GUI.profile.d))/GUI.profile.nu))],'Linear range'); % Some legends must be reset due to error
    end
    
    function plotRSS()        
        [z_plus_interp, RSS_calc] = calcCSS(GUI.profile);
        varform = size(GUI.profile.u.mean);
        z_plot = reshape((GUI.profile.z-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d),varform);
        plotMask =  GUI.cssInd(1) < z_plot & z_plot < GUI.cssInd(2);
        RSS_plus = -GUI.profile.uw./GUI.profile.utau.^2;
        if GUI.DSSexist
            DSS_plus = -GUI.profile.DSS./GUI.profile.utau.^2;
            plot(GUI.GUI_RSS,z_plot,RSS_plus+DSS_plus,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','b','MarkerFaceColor','b');
            set(GUI.GUI_RSS,'NextPlot','add');
            SSoptimizationDataPoints = RSS_plus(plotMask)+DSS_plus(plotMask);
            plot(GUI.GUI_RSS,z_plot(plotMask),SSoptimizationDataPoints,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','r','MarkerFaceColor','r');
            ylabeltext = '$$-\left \langle \overline{{u}''{w}''} \right \rangle^+-\left \langle \tilde{u}\tilde{w} \right \rangle^+$$';
        else
            plot(GUI.GUI_RSS,z_plot,RSS_plus,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','b','MarkerFaceColor','b');
            set(GUI.GUI_RSS,'NextPlot','add');
            SSoptimizationDataPoints = RSS_plus(plotMask);
            plot(GUI.GUI_RSS,z_plot(plotMask),SSoptimizationDataPoints,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','r','MarkerFaceColor','r');
            ylabeltext = '$$-\left ( \overline{{u}''{w}''} \right )^+$$';
        end

        plotSupress = z_plus_interp > (GUI.profile.z(1)-GUI.profile.d)*GUI.profile.utau/GUI.profile.nu;
        plot(GUI.GUI_RSS,z_plus_interp(plotSupress)/GUI.profile.utau*GUI.profile.nu/(GUI.profile.delta-GUI.profile.d),RSS_calc(plotSupress),'k','LineStyle','-','LineWidth',1.2,'Marker','none','MarkerSize',16);
        set(GUI.GUI_RSS,'NextPlot','replacechildren');
        GUI.GUI_RSS_Legend = legend(GUI.GUI_RSS,'Data point','CSS range','CSS equation','Location','southwest');
        xlim(GUI.GUI_RSS,[0 1]);
        ylabel(GUI.GUI_RSS,ylabeltext,'interpreter','latex','FontSize',18);
        
        % Calculate Least Squares for CSS formula
        utauRange = linspace(GUI.utau_slider.Min,GUI.utau_slider.Max,GUI.LSres);
        z0Range = linspace(GUI.z0_slider.Min,GUI.z0_slider.Max,GUI.LSres);
        
        for j = 1:length(utauRange)
            for k = 1:length(z0Range)
                GUI.RMS_css(j,k) = errorCSS(utauRange(j),z0Range(k));
            end
        end
    end

    function Pi = calcPi()
        delta_nu = GUI.profile.nu/GUI.profile.utau;
        z_plus = (GUI.profile.z-GUI.profile.d)./delta_nu;
        u_plus = GUI.profile.u.mean./GUI.profile.utau;
        u_plus_fit = 1/GUI.profile.kvk*log(z_plus.*delta_nu./GUI.profile.z0);
        infMask = ~isinf(u_plus_fit);
        Pi = max(reshape(u_plus(infMask)-u_plus_fit(infMask),[],1))*GUI.profile.kvk/2;
        if isfield(GUI,'Pi_editBox')
            set(GUI.Pi_editBox(2),'String',num2str(Pi));
            if get(GUI.Pi_radio(2),'Enable')
                SL = get(GUI.Pi_slider,{'Min','Value','Max'});
                if GUI.profile.Pi < SL{1}
                    GUI.profile.Pi = SL{1};
                elseif GUI.profile.Pi > SL{3}
                    GUI.profile.Pi = SL{3};
                end
                set(GUI.Pi_slider,'Value',Pi);
            end
        end
        GUI.profile.Pi = Pi;
    end

    function plotLinDiagnostic()
        % Plots the linear diagnostic function for the profile
        z_delta = transpose((GUI.profile.z-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d));
        z_plus = (GUI.profile.z-GUI.profile.d).*(GUI.profile.utau/GUI.profile.nu);
        u_plus = GUI.profile.u.mean./GUI.profile.utau;
        if size(z_plus)==size(u_plus)
            phi = diffxy(z_plus,u_plus).*z_plus;
        else
            phi = diffxy(z_plus,u_plus).*transpose(z_plus);
        end
        
        plot(GUI.GUI_linDiagnostic,z_delta,phi,'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','b','MarkerFaceColor','b');
        set(GUI.GUI_linDiagnostic,'NextPlot','add');
        plot(GUI.GUI_linDiagnostic,z_delta(GUI.profile.linInd),phi(GUI.profile.linInd),'LineStyle','none','LineWidth',1.2,'Marker','.','MarkerSize',16,'MarkerEdgeColor','r','MarkerFaceColor','r');
        plot(GUI.GUI_linDiagnostic,[z_delta(1) z_delta(length(z_delta))],[0.9/GUI.profile.kvk 0.9/GUI.profile.kvk],'k-');
        plot(GUI.GUI_linDiagnostic,[z_delta(1) z_delta(length(z_delta))],[1.1/GUI.profile.kvk 1.1/GUI.profile.kvk],'k-');
        set(GUI.GUI_linDiagnostic,'NextPlot','replacechildren');
        xlim(GUI.GUI_linDiagnostic,[z_delta(1) z_delta(length(z_delta))]);
        ylim(GUI.GUI_linDiagnostic,[0 7]);
        legend(GUI.GUI_linDiagnostic,'Data point','Linear range','Location','best');
    end

    function plotShear()
        tau_RSS = -GUI.profile.rho.*GUI.profile.uw;
        tau_visc = GUI.profile.mu.*diffxy(GUI.profile.z,GUI.profile.u.mean);
        tau = tau_visc + tau_RSS;
        if GUI.DSSexist
            tau = tau - GUI.profile.rho.*GUI.profile.DSS;
        end
        tau_w = GUI.profile.rho*GUI.profile.utau^2;
        plot(GUI.GUI_totalShear,GUI.profile.z/(GUI.profile.delta-GUI.profile.d),tau,'b.',[GUI.profile.z(1) GUI.profile.z(length(GUI.profile.z))]./(GUI.profile.delta-GUI.profile.d),[tau_w tau_w],'k-');
        ylim(GUI.GUI_totalShear,[min(tau)-0.25*tau_w 1.25*tau_w]);
        legend(GUI.GUI_totalShear,'\tau','\tau_w','Location','southwest');
    end

    function calcRMS_dz()
        u_fit = @(z,z0) 1/GUI.profile.kvk*log(z/z0);
        nPoints = length(GUI.profile.linInd);
        u = reshape(GUI.profile.u.mean(GUI.profile.linInd),[],1);
        z = reshape(GUI.profile.z(GUI.profile.linInd),[],1);
        RMS = @(x) sqrt(sum((u./x(1)-u_fit(z-x(3),x(2))).^2)./nPoints);
        z0Range = linspace(GUI.z0_slider.Min,GUI.z0_slider.Max,GUI.LSres);
        dRange = linspace(GUI.d_slider.Min,GUI.d_slider.Max,GUI.LSres);
        GUI.RMS_dz = NaN(length(dRange),length(z0Range));
        jEnd = min([length(dRange) find(z(1)<dRange,1,'first')-1]);
        for j = 1:jEnd
            for k = 1:length(z0Range)
                GUI.RMS_dz(j,k) = RMS([GUI.profile.utau z0Range(k) dRange(j)]);
            end
        end
    end

    function calcRMS_du()
        u_fit = @(z,z0) 1/GUI.profile.kvk*log(z/z0);
        nPoints = length(GUI.profile.linInd);
        u = reshape(GUI.profile.u.mean(GUI.profile.linInd),[],1);
        z = reshape(GUI.profile.z(GUI.profile.linInd),[],1);
        RMS = @(x) sqrt(sum((u./x(1)-u_fit(z-x(3),x(2))).^2)./nPoints);
        utauRange = linspace(GUI.utau_slider.Min,GUI.utau_slider.Max,GUI.LSres);
        dRange = linspace(GUI.d_slider.Min,GUI.d_slider.Max,GUI.LSres);
        GUI.RMS_du = NaN(length(dRange),length(utauRange));
        jEnd = min([length(dRange) find(z(1)<dRange,1,'first')-1]);
        for j = 1:jEnd
            for k = 1:length(utauRange)
                GUI.RMS_du(j,k) = RMS([utauRange(k) GUI.profile.z0 dRange(j)]);
            end
        end
    end

    function calcRMS_uz()
        u_fit = @(z,z0) 1/GUI.profile.kvk*log(z/z0);
        nPoints = length(GUI.profile.linInd);
        u = reshape(GUI.profile.u.mean(GUI.profile.linInd),[],1);
        z = reshape(GUI.profile.z(GUI.profile.linInd),[],1);
        RMS = @(x) sqrt(sum((u./x(1)-u_fit(z-x(3),x(2))).^2)./nPoints);
        utauRange = linspace(GUI.utau_slider.Min,GUI.utau_slider.Max,GUI.LSres);
        z0Range = linspace(GUI.z0_slider.Min,GUI.z0_slider.Max,GUI.LSres);
        GUI.RMS_uz = NaN(length(utauRange),length(z0Range));
        for j = 1:length(utauRange)
            for k = 1:length(z0Range)
                GUI.RMS_uz(j,k) = RMS([utauRange(j) z0Range(k) GUI.profile.d]);
            end
        end
    end

    function [z0] = optimize_z(varargin)
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        
        nPoints = length(GUI.profile.linInd);
        u = reshape(GUI.profile.u.mean(GUI.profile.linInd),[],1);
        z = reshape(GUI.profile.z(GUI.profile.linInd),[],1);
        RMS = @(x) sqrt(sum((u./GUI.profile.utau-1./GUI.profile.kvk.*log((z-GUI.profile.d)./x)).^2)./nPoints);
        
        xTol = 0.01*(GUI.z0_slider.Max-GUI.z0_slider.Min);
        
        options = optimset('TolX', xTol);
        z0 = fminbnd(@(x) RMS(x),GUI.z0_slider.Min,GUI.z0_slider.Max,options);
        
        set(GUI.z0_slider,'Value',z0);
        set(GUI.z0_editBox(2),'String',num2str(z0));
        GUI.profile.z0 = z0;
        
        calcRMS_du();
        plotFit();
    end

    function [d] = optimize_d(varargin)
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        
        nPoints = length(GUI.profile.linInd);
        u = reshape(GUI.profile.u.mean(GUI.profile.linInd),[],1);
        z = reshape(GUI.profile.z(GUI.profile.linInd),[],1);
        RMS = @(x) sqrt(sum((u./GUI.profile.utau-1./GUI.profile.kvk.*log((z-x)./GUI.profile.z0)).^2)./nPoints);
        
        xTol = 0.01*(GUI.d_slider.Max-GUI.d_slider.Min);
        
        options = optimset('TolX', xTol);
        d = fminbnd(@(x) RMS(x),GUI.d_slider.Min,GUI.d_slider.Max,options);
        
        set(GUI.d_slider,'Value',d);
        set(GUI.d_editBox(2),'String',num2str(d));
        GUI.profile.d = d;
        
        calcRMS_uz();
        plotShear();
        plotFit();
    end

    function [d, z0] = optimize_dz(varargin)
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        u_plus = GUI.profile.u.mean(GUI.profile.linInd)./GUI.profile.utau;
        u_plus = u_plus';
        linReg = fitlm(GUI.profile.z(GUI.profile.linInd),exp(GUI.profile.kvk.*u_plus),'linear');
        z0 = 1/linReg.Coefficients{2,1};
        d = -z0*linReg.Coefficients{1,1};
        
        d_SL = get(GUI.d_slider,{'Min','Value','Max'});
        if d < d_SL{1}
            set(GUI.d_slider,'Value',d_SL{1});
            set(GUI.d_editBox(2),'String',num2str(d_SL{1}));
            GUI.profile.d = d_SL{1};
            z0 = optimize_z();
            calcRMS_uz();
        elseif d > d_SL{3}
            set(GUI.d_slider,'Value',d_SL{3});
            set(GUI.d_editBox(2),'String',num2str(d_SL{3}));
            GUI.profile.d = d_SL{3};
            z0 = optimize_z();
            calcRMS_uz();
        else
            set(GUI.d_slider,'Value',d);
            set(GUI.d_editBox(2),'String',num2str(d));
            GUI.profile.d = d;
            calcRMS_uz();
        end
        
        z0_SL = get(GUI.z0_slider,{'Min','Value','Max'});
        if z0 < z0_SL{1}
            set(GUI.z0_slider,'Value',z0_SL{1});
            set(GUI.z0_editBox(2),'String',num2str(z0_SL{1}));
            GUI.profile.z0 = z0_SL{1};
            optimize_d();
            calcRMS_du();
        elseif z0 > z0_SL{3}
            set(GUI.z0_slider,'Value',z0_SL{3});
            set(GUI.z0_editBox(2),'String',num2str(z0_SL{3}));
            GUI.profile.z0 = z0_SL{3};
            optimize_d();
            calcRMS_du();
        else
            set(GUI.z0_slider,'Value',z0);
            set(GUI.z0_editBox(2),'String',num2str(z0));
            GUI.profile.z0 = z0;
            calcRMS_du();
        end
        
        update_linInd();
        plotShear();
        plotFit();
    end

    function [utau] = optimize_css(varargin)
        set(GUI.Pi_editBox(2),'String','Calculating...');
        drawnow;
        xTol = (GUI.utau_slider.Max-GUI.utau_slider.Min)/100;
        options = optimset('TolX', xTol);
        utau = fminbnd(@(x) errorCSS(x,GUI.profile.z0),GUI.utau_slider.Min,GUI.utau_slider.Max,options);
        set(GUI.utau_slider,'Value',utau);
        set(GUI.utau_editBox(2),'String',num2str(utau));
        GUI.profile.utau = utau;
        calcRMS_dz();
        plotShear();
        plotFit();
    end
    
    function [z_plus_interp, cssProfile] = calcCSS(profile,calcMask)
        if ~exist('calcMask','var') % calcmask avoids calculation of css beyond what is necessary
            calcMask = true(size(profile.u.mean));
        end
        
        % Cubic interpolation of profile to z-d=0
        u_plus = profile.u.mean(calcMask)/profile.utau;
        varform = size(u_plus);
        z_plus = (profile.z(calcMask)-profile.d)*profile.utau/profile.nu;
        z_plus_interp = horzcat(profile.z0*profile.utau/profile.nu:20:z_plus(1),z_plus); % adds a near wall region for interpolation to z-d=z0plus

        if z_plus(1) ~=0 % applies pchip interpolation to extrapolate points between z-d=z0plus and nearest data. See Volino and Schultz (2018) for analysis of possible effects.
            u_plus_interp = interp1(horzcat(0,z_plus),horzcat(0,u_plus),z_plus_interp,'pchip');
        else
            u_plus_interp = interp1(z_plus,u_plus,z_plus_interp,'pchip');
        end
        
        % Calculate CSS
        H = profile.deltaStar/profile.theta;
        utau_Ue = profile.utau/profile.Ue;
        % Cf = 2*(utau_Ue)^2;
        s = utau_Ue^-1;        
        % I = sum(trapz(profile.z,((profile.Ue-profile.u.mean)/profile.utau).^2))/sum(trapz(profile.z,(profile.Ue-profile.u.mean)/profile.utau)); % By Clauser (1954) definition
        I = s*(H-1)/H; % By definitions, faster to avoid integral calcs
        K = 2*profile.Pi/profile.kvk-log((1+profile.Pi)/profile.kvk)/profile.kvk; % K with actual Pi
        % K = 2*0.55/profile.kvk-log((1+0.55)/profile.kvk)/profile.kvk; % K with Pi=0.55
        C_f_gradTerm = -exp(-profile.kvk*(s-K))./(profile.z0*s.^2).*(1+(s-I).*(profile.kvk-2./s)).^-1;        
        
        dtheta_dx = (utau_Ue)^2 - (2*profile.theta + profile.deltaStar)*profile.Ue/profile.nu*profile.Kp;
        wakeMask = z_plus_interp/profile.utau*profile.nu/(profile.delta-profile.d) > GUI.linRange(1);
        negMask = z_plus_interp <= 0;
        if any(negMask)
            warning('Some data points are below d and ignored for CSS');
            negMask = ~negMask;
            z_plus_interp = z_plus_interp(negMask);
            u_plus_interp = u_plus_interp(negMask);
            wakeMask = wakeMask(negMask);
        end
        dup_dx = -(profile.Pi/(profile.kvk))*z_plus_interp*(dtheta_dx/profile.theta+profile.Ue/profile.utau*C_f_gradTerm).*diffxy(z_plus_interp,u_plus_interp-(1/profile.kvk)*log(z_plus_interp*profile.nu/(profile.z0*profile.utau)));
        
        cssProfile = 1 - diffxy(z_plus_interp,u_plus_interp)...
                        - profile.Kp/utau_Ue^3*z_plus_interp...
                        + (profile.Kp/utau_Ue + profile.nu/(profile.Ue*utau_Ue^2)*C_f_gradTerm)*cumtrapz(z_plus_interp,u_plus_interp.^2)...
                        + (profile.nu/profile.utau)*wakeMask.*(2*cumtrapz(z_plus_interp,u_plus_interp.*dup_dx)-u_plus_interp.*cumtrapz(z_plus_interp,dup_dx));
                        % Term V neglected
                        % Term VI neglected
    end

    function [error] = errorCSS(utau,z0)
        z_plot = (GUI.profile.z-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d);
        calcMask =  z_plot < GUI.cssInd(2); % Creates mask so that profile is not calculated beyond last fitted index
        plotMask =  GUI.cssInd(1) < z_plot & calcMask;
        calcMask(sum(calcMask)+1) = true;
        RSS_plus = -GUI.profile.uw./utau.^2;
        if GUI.DSSexist
            DSS_plus = -GUI.profile.DSS./utau.^2;
            SSoptimizationDataPoints = RSS_plus(plotMask)+DSS_plus(plotMask);
        else
            SSoptimizationDataPoints = RSS_plus(plotMask);
        end
        
        % Calculate Least Squares for CSS formula
        errProfile = GUI.profile;
        errProfile.utau = utau;
        errProfile.z0 = z0;
        [z_plus, cssProfile] = calcCSS(errProfile, calcMask);
        eqnMask = false(size(z_plus));
        tempMask = plotMask(calcMask);
        eqnMask(length(eqnMask)-length(tempMask)+1:end) = tempMask;
        nPoints = sum(plotMask);
        error = sqrt(sum((SSoptimizationDataPoints-cssProfile(eqnMask)).^2)./nPoints);
    end

    function update_linInd()
        z_delta = transpose((GUI.profile.z-GUI.profile.d)/(GUI.profile.delta-GUI.profile.d));
        GUI.profile.linInd =  find(GUI.linRange(1) < z_delta & z_delta < GUI.linRange(2));
    end

    function optimize_auto(varargin)
        utau_last = -1000;
        z0_last = -1000;
        n = 0;
        while (round(z0_last,3,'significant') ~= round(GUI.profile.z0,3,'significant') || round(utau_last,3,'significant') ~= round(GUI.profile.utau,3,'significant')) && n < 10
            n = n+1;
            set(GUI.optimize_auto_button,'String',num2str(n));
            drawnow();
            utau_last = GUI.profile.utau;
            z0_last = GUI.profile.z0;
            optimize_css();
            drawnow();
            if get(GUI.fix_d_toggle,'Value')
                optimize_z();
            elseif get(GUI.smoothwallAuto_toggle,'Value')
                GUI.profile.z0 =  GUI.profile.nu*exp(-GUI.profile.A*GUI.profile.kvk)/GUI.profile.utau;
                if GUI.profile.z0 >= get(GUI.z0_slider,'Min') && GUI.profile.z0 <= get(GUI.z0_slider,'Max')
                    set(GUI.z0_slider,'Value',GUI.profile.z0);
                    set(GUI.z0_editBox(2),'String',num2str(GUI.profile.z0));
                elseif GUI.profile.z0 < get(GUI.z0_slider,'Min')
                    GUI.profile.z0 = get(GUI.z0_slider,'Min');
                    set(GUI.z0_slider,'Value',GUI.profile.z0);
                    set(GUI.z0_editBox(2),'String',num2str(GUI.profile.z0));
                else
                    GUI.profile.z0 = get(GUI.z0_slider,'Max');
                    set(GUI.z0_slider,'Value',GUI.profile.z0);
                    set(GUI.z0_editBox(2),'String',num2str(GUI.profile.z0));
                end
                calcRMS_du();
                plotFit();
            else
                optimize_dz();
            end
        end
        set(GUI.optimize_auto_button,'String','Auto');
%         soundVec = load('train.mat');
%         sound(repmat(soundVec.y/2,1,1),soundVec.Fs);
    end

    function GUIfigClosereq(~,~)
        % Save figure
        print(['CSS GUI - ' strrep(strrep(GUI.profile.name,'_',' '),',','.') '.png'],'-dpng','-r0');
%         print(['CSS GUI - ' strrep(strrep(GUI.profile.name,'_',' '),',','.') '_kvk' num2str(GUI.profile.kvk*1000,'%03.0f') '_linRange' num2str(GUI.linRange(1)*100,'%02.0f') num2str(GUI.linRange(2)*100,'%02.0f') '_cssRange' num2str(GUI.cssInd(1)*100,'%02.0f') num2str(GUI.cssInd(2)*100,'%02.0f') '.png'],'-dpng','-r0');
        % Delete figure
        delete(gcf);
    end

end