classdef profileClass
    properties
        x       % streamwise axis (m)
        y       % spanwise axis (m)
        z       % wall-normal axis (m)
        u       % steamwise velocity struct (m/s)
        w       % wall-normal velocity struct (m/s)
        uw      % time average of u'w' (m/s)^2
        nUW     % number of valid measurements for uw
        DSS     % dispersive shear stress (m/s)^2
        nCoinc  % number of coincident measurements
        Ue      % freestream velocity (m/s)
        delta   % delta 99 (m)
        theta   % momentum thickness (m)
        deltaStar % displacement thickness (m)
        utau    % friction velocity (m/s)
        d       % zero-plane displacement (m)
        z0      % roughness length (m)
        Pi      % wake strength parameter
        kvk     % von Karman constant
        A       % smooth-wall origin
        rho     % fluid density (kg/m^3)
        mu      % dynamic viscosity (kg/m-s)
        nu      % kinematic viscosity (m^2/s)
        linInd  % linear indexes
        Kp      % acceleration parameter
        name    % profile name
    end
    methods
        function struct = profile2struct(profile)
            struct.x = reshape(cast(profile.x,'double'),[],1);
            struct.y = reshape(cast(profile.y,'double'),[],1);
            struct.z = reshape(cast(profile.z,'double'),[],1);
            struct.ua = reshape(profile.u.mean,[],1);
            struct.urms = reshape(profile.u.rms,[],1);
            struct.uskew = reshape(profile.u.skewness,[],1);
            struct.uflat = reshape(profile.u.flatness,[],1);
            struct.un = reshape(profile.u.n,[],1);
            struct.wa = reshape(profile.w.mean,[],1);
            struct.wrms = reshape(profile.w.rms,[],1);
            struct.wskew = reshape(profile.w.skewness,[],1);
            struct.wflat = reshape(profile.w.flatness,[],1);
            struct.wn = reshape(profile.w.n,[],1);
            struct.uw = reshape(profile.uw,[],1);
            struct.nUW = reshape(profile.nUW,[],1);
            struct.DSS = reshape(profile.DSS,[],1);
            struct.nCoinc = reshape(profile.nCoinc,[],1);
            struct.Ue = profile.Ue;
            struct.delta = cast(profile.delta,'double');
            struct.utau = profile.utau;
            struct.d = profile.d;
            struct.z0 = profile.z0;
            struct.Pi = profile.Pi;
            struct.kvk = profile.kvk;
            struct.rho = profile.rho;
            struct.mu = profile.mu;
            struct.nu = profile.nu;
            struct.linInd = reshape(profile.linInd,[],1);
            struct.Kp = profile.Kp;
            struct.name = profile.name;
        end
        
        function profile = calcUe(profile)
            du_dz = diffxy(profile.z,profile.u.mean);
            testInd = round(length(profile.z)/2):length(profile.z);
            while du_dz(testInd(1)) > 5e-4 && length(testInd) > 1
                testInd = testInd(2):length(profile.z);
            end
            profile.Ue = mean(profile.u.mean(testInd));
        end
        
        function profile = calcDelta(profile)
            if not(isempty(profile.Ue))
                mask = ~isnan(profile.u.mean);
                u_mean = profile.u.mean(mask);
                checkInc = find(diff(u_mean) <= 0);
                while ~isempty(checkInc)
                    if profile.u.mean(checkInc(1)) > 0.99*profile.Ue
                        mask(checkInc(1):length(mask)) = 0;
                        checkInc = [];
                    else
                        mask(1:checkInc(1)) = 0;
                        if length(checkInc) > 1
                            checkInc = checkInc(2:length(checkInc));
                        else
                            checkInc = [];
                        end
                    end
                end
                u_mean = profile.u.mean(mask);
                profile.delta = interp1(u_mean,profile.z(mask),0.99*profile.Ue);
            end
        end
        
        function theta = calcTheta(profile,varargin)
            p = inputParser;
            defaultInterp = 'linear';
            expectedInterp = {'none','linear','pchip'};
            defaultPower = 1;
            
            addRequired(p,'profile',@(x) isa(x,'profileClass'));
            addOptional(p,'interp',defaultInterp,@(x) any(validatestring(x,expectedInterp)));
            addParameter(p,'powerlaw',defaultPower,@isnumeric);
            
            parse(p,profile,varargin{:});
            if not(isempty(profile.Ue) && length(profile.z)<2 && length(profile.z)==length(profile.u.mean))
                theta = sum(trapz(profile.z,profile.u.mean.*(profile.Ue-profile.u.mean)))/profile.Ue^2;
                switch p.Results.interp
                    case 'none'
                        % Do nothing
                    case 'pchip'
                        pp = pchip(horzcat(0,profile.z(1:2)),horzcat(0,profile.u.mean(1:2)));
                        nearWallIntegral = (profile.Ue*(pp.coefs(1,1)*pp.breaks(2)^4/4+pp.coefs(1,2)*pp.breaks(2)^3/3)-(pp.coefs(1,1)^2*pp.breaks(2)^7/7+pp.coefs(1,1)*pp.coefs(1,2)*pp.breaks(2)^6/3+pp.coefs(1,2)^2*pp.breaks(2)^5/5))/profile.Ue^2;
                        theta = nearWallIntegral + theta;
                    otherwise
                        uRatio = profile.u.mean(1)/profile.Ue;
                        z1 = profile.z(1);
                        nearWallIntegral = uRatio*z1/(p.Results.powerlaw+1) - uRatio^2*z1/(2*p.Results.powerlaw+1);
                        theta = nearWallIntegral + theta;
                end
            else
                error('Input profile does not contain required properties');
            end
        end
        
        function deltaStar = calcDeltaStar(profile,varargin)
            p = inputParser;
            defaultInterp = 'linear';
            expectedInterp = {'none','linear','pchip'};
            defaultPower = 1;
            
            addRequired(p,'profile',@(x) isa(x,'profileClass'));
            addOptional(p,'interp',defaultInterp,@(x) any(validatestring(x,expectedInterp)));
            addParameter(p,'powerlaw',defaultPower,@isnumeric);
            
            parse(p,profile,varargin{:});
            if not(isempty(profile.Ue) && length(profile.z)<2 && length(profile.z)==length(profile.u.mean))
                deltaStar = sum(trapz(profile.z,(profile.Ue-profile.u.mean)))/profile.Ue;
                switch p.Results.interp
                    case 'none'
                        % Do nothing
                    case 'pchip'
                        pp = pchip(horzcat(0,profile.z(1:2)),horzcat(0,profile.u.mean(1:2)));
                        nearWallIntegral = (profile.Ue*pp.breaks(2)-pp.coefs(1,1)*pp.breaks(2)^4/4-pp.coefs(1,2)*pp.breaks(2)^3/3)/profile.Ue;
                        deltaStar = nearWallIntegral + deltaStar;
                    otherwise
                        uRatio = profile.u.mean(1)/profile.Ue;
                        z1 = profile.z(1);
                        nearWallIntegral = z1*(1-uRatio/(p.Results.powerlaw+1));
                        deltaStar = nearWallIntegral + deltaStar;
                end
            else
                error('Input profile does not contain required properties');
            end
        end
        
        function ks = calcKs(profile)
            ks = profile.z0.*exp(8.5*profile.kvk);
        end
        
        function delta_Uplus = calcDeltaUplus(profile,A)
            delta_Uplus = (1/profile.kvk)*log(profile.z0*profile.utau/profile.nu)+A;
        end
        
        function profile = csv2profile(profile,folder,name)
            if exist([folder name '.csv'],'file')
                csvData = csvread([folder name '.csv'],1,0);
                profile.z = reshape(csvData(:,1),1,[]);
                profile.u.mean = reshape(csvData(:,2),1,[]);
                profile.uw = reshape(csvData(:,3),1,[]);
                if csvData(:,4) ~= zeros(length(csvData(:,4)),1)
                    profile.DSS = reshape(csvData(:,4),1,[]);
                end
                profile.rho = csvData(1,5);
                profile.nu = csvData(1,6);
                profile.Ue = csvData(1,7);
                profile.delta = csvData(1,8);
                profile.utau = csvData(1,9);
                profile.z0 = csvData(1,10);
                profile.d = csvData(1,11);
                profile.Kp = csvData(1,12);
                
                profile.mu = profile.rho*profile.nu;
                
                profile.name = name;
            else
                error(['No CSV file found.\n' folder name '.csv not found.']);
            end
        end
    end
end